﻿using Kivii.DataAnnotations;
using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.OnlineCustomization.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.OnlineCustomization.Entities
{
    public class OnlineOrder :IEntityIsThirdReportOrder
    {
        #region field
        [ApiMember(Description = "订单号")]
        public string OrderId { get; set; }

        /// <summary>
        /// 购物车内不同商品根据分类进行分组
        /// </summary>
        [ApiMember(Description = "分类")]
        public string Category { get; set; }

        [ApiMember(Description = "订单总金额")]
        [DecimalLength(10, 2)]
        public decimal TotalAmount { get; set; }

        [ApiMember(Description = "所属Kvid")]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [ApiMember(Description = "买家单位")]
        public string BuyerCompony { get; set; }

        [ApiMember(Description = "买家名称")]
        public string BuyerName { get; set; }

        [ApiMember(Description = "买家手机号")]
        public string BuyerMobile { get; set; }

        [ApiMember(Description = "省份")]
        public string Province { get; set; }

        [ApiMember(Description = "城市")]
        public string City { get; set; }

        [ApiMember(Description = "区县")]
        public string District { get; set; }

        [ApiMember(Description = "详细地址")]
        public string Address { get; set; }

        [ApiMember(Description = "备注")]
        public string Remark { get; set; }

        /// <summary>
        /// 订单关联的报告地址
        /// </summary>
        public List<string> Urls { get; set; }

        #region 创建/更新人
        [ApiMember(Description = "创建人Kvid")]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        public string UpdaterName { get; set; }
        #endregion
        //
        // 摘要:
        //     附加属性，以键/值的方式存在,数据将会以XML形式存入数据表
        [ApiMember(Description = "附加属性，以键/值的方式存在")]
        [Ignore]
        public Dictionary<string, string> Metadata { get; set; }
        //
        // 摘要:
        //     主键，系统中所有的实体表中都以Kvid为主键
        [ApiMember(Description = "唯一编号,实体对象的唯一编号，GUID类型，当创建实体对象时，如不传入值，系统将自动分配")]
        [IgnoreUpdate]
        [PrimaryKey]
        public Guid Kvid { get; set; }
        //
        // 摘要:
        //     创建时间，创建数据时自动赋值，取当前数据库的时间（并非服务器时间） 除非是同步数据时，可以手动指定（设置实体Tag=KiviiContext.EntityTagForDbSync）
        [ApiMember(Description = "创建时间,实体对象的创建时间，系统自动生成。")]
        [CurrentDateTime]
        [IgnoreUpdate]
        [Required]
        public DateTime CreateTime { get; set; }
        //
        // 摘要:
        //     最后更新时间，更新数据时自动赋值，取当前数据库的时间（并非服务器时间） 除非是同步数据时，可以手动指定（设置实体Tag=KiviiContext.EntityTagForDbSync）
        [ApiMember(Description = "更新时间,实体对象的最后更新时间，系统自动生成。")]
        [CurrentDateTime]
        [Required]
        public DateTime UpdateTime { get; set; }
        //
        // 摘要:
        //     数据的状态。实体对象的状态，正常为大于零的一个值，等于零一般表示实体对象在临时状态，小于零一般表示为实体对象已标记为删除状态，不会被查询到。 当需要使用此字段做其它用途时，可以在继承类中overried
        //     此字段不能通过Restful接口更新，只能在代码中更新
        [ApiMember(Description = "数据状态,实体对象的状态，正常为大于零的一个值，等于零一般表示实体对象在临时状态，小于零一般表示为实体对象已标记为删除状态，不会被查询到。")]
        [Default(0)]
        [GlobalFilter(">-1")]
        [InternalSetter]
        [NonClusteredIndex]
        public int Status { get; set; }

        [Ignore]
        public OrderStatusType StatusType
        {
            get => (OrderStatusType)Status;
            set => Status = (int)value;
        }
        [Ignore]
        public List<OnlineOrderItem> Items { get; set; }
        #endregion

        #region Interface
        public Report OnInstanceThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return null;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return null;
            if (third.ThirdType != typeof(OnlineOrder).FullName) return null;
            if (third.Metadata == null) return null;
            Console.WriteLine("-----------------"+ third.Metadata);
            var order = third.Metadata.DicToObject();
            Console.WriteLine("-----------------222222222222222");
            var report = new Report();
            report.Metadata = new Dictionary<string, string>();

            // 安全获取字典值的辅助方法
            string GetMetadataValue(string key) => order.Items?.Any() == true && order.Items[0].Metadata?.ContainsKey(key) == true ? order.Items[0].Metadata[key] : "";

            // 委托方信息
            report.DemanderName = GetMetadataValue("DemanderName");
            report.DemanderContactName = GetMetadataValue("DemanderContactName");
            report.DemanderContactNumber = GetMetadataValue("DemanderContactNumber");
            report.Metadata["DemanderContactOfficePhone"] = GetMetadataValue("DemanderContactOfficePhone");
            report.Metadata["DemanderEmail"] = GetMetadataValue("DemanderEmail");
            report.DemanderContactAddress = GetMetadataValue("DemanderFullAddress");
            report.Metadata["DemanderPostcode"] = GetMetadataValue("DemanderZipCode");
            
            // 生产方信息
            report.SampleManufacturerName = GetMetadataValue("SampleManufacturerName");
            report.SampleManufacturerContactName = GetMetadataValue("SampleManufacturerContactName");
            report.SampleManufacturerContactNumber = GetMetadataValue("SampleManufacturerContactNumber");
            report.Metadata["SampleManufacturerContactOfficePhone"] = GetMetadataValue("SampleManufacturerContactOfficePhone");
            report.Metadata["SampleManufacturerEmail"] = GetMetadataValue("SampleManufacturerEmail");
            report.SampleManufacturerAddress = GetMetadataValue("ManufacturerFullAddress");

            // 付款单位
            report.PayerName = GetMetadataValue("FullName");
            
            // 样品信息
            report.SampleName = GetMetadataValue("SampleName");
            report.SampleModel = GetMetadataValue("SampleModel");
            report.SampleBatchNumber = GetMetadataValue("SampleBatchNumber");
            report.Metadata["SampleDescriptionEx"] = GetMetadataValue("SampleDescriptionEx");
            report.SampleGrade = GetMetadataValue("SampleGrade");
            report.SampleGradeSafety = GetMetadataValue("SampleGradeSafety");
            report.Metadata["SampleSelfColor"] = GetMetadataValue("SampleSelfColor");
            report.Metadata["FibreContent"] = GetMetadataValue("FibreContent");
            report.Metadata["TechParameter"] = GetMetadataValue("TechParameter");
            report.SampleQuantityUnit = GetMetadataValue("SampleQuantityUnit");
            
            // 备注信息
            report.Remark = GetMetadataValue("remark");
            
            // 检测项目Kvids - 将字符串格式转换为List<Guid>
            var detectionKvidsStr = GetMetadataValue("DetectionKvids");
            Console.WriteLine(detectionKvidsStr);
            if (!string.IsNullOrEmpty(detectionKvidsStr))
            {
                try
                {
                    // 移除开头和结尾的方括号
                    var cleanStr = detectionKvidsStr.Trim('[', ']');
                    if (!string.IsNullOrEmpty(cleanStr))
                    {
                        // 按逗号分割并转换为Guid列表
                        report.DetectionKvids = cleanStr.Split(',')
                            .Where(s => !string.IsNullOrWhiteSpace(s))
                            .Select(s => Guid.Parse(s.Trim()))
                            .ToList();
                    }
                    else
                    {
                        report.DetectionKvids = new List<Guid>();
                    }
                }
                catch (Exception)
                {
                    // 如果转换失败，设置为空列表
                    report.DetectionKvids = new List<Guid>();
                }
            }
            else
            {
                report.DetectionKvids = new List<Guid>();
            }
            
            return report;
        }

        public object OnPullThirdReport()
        {
            var rtns = new RestfulQueryResponse<ThirdReportOrder>();
            var client = new JsonHttpClient(Configs._defaultUrl);
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(OnlineOrder).FullName);

            try
            {
                var response = client.PullOrders(Configs._token, Configs._secret, maxTranslateTime);

                rtns.Results = new List<ThirdReportOrder>();
                if (response == null) return rtns;
                if (response.Results.IsNullOrEmpty()) return rtns;
                var results = response.Results.AcceptThirdContractOrder(null, null, conn);

                rtns.Results.AddRange(results);
                rtns.Total = rtns.Results.Count;
                var thread = KiviiContext.NewThread(() =>
                {
                    //没有关联报告的 自动执行符合条件的关联
                    OrderExtension.LinkReports();
                });
                thread.Start();
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return rtns;
        }

        public bool OnRejectThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(OnlineOrder).FullName) return false;
            //订单还未受理的才退回 否则不能退回
            if (third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return false;
            var client = new JsonHttpClient(Configs._defaultUrl);
            try
            {
                var resp = client.RejectOrder(third.SerialNumber, Configs._token, Configs._secret);
                
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public bool OnUploadThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(OnlineOrder).FullName) return false;
            //订单还未受理的 则自动去找相关报告进行关联
            if (!third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return true;
            var thirdDetails = conn.Select<ThirdReportOrderDetail>(o => o.ThirdOrderKvid == third.Kvid);
            if (thirdDetails.IsNullOrEmpty()) return false;
            var thirdDetailDis = new List<ThirdReportOrderDetail>();
            foreach (var item in thirdDetails)
            {
                if (!thirdDetailDis.Exists(o => o.ReportKvid == item.ReportKvid)) thirdDetailDis.Add(item);
                continue;
            }
            if (thirdDetailDis.IsNullOrEmpty()) return false;
            var reports = conn.SelectByIds<Report>(thirdDetailDis.ConvertAll(o => o.ReportKvid));
            if (reports.IsNullOrEmpty()) return false;
            if (reports.Exists(o => o.Status <= (int)ReportStatusType.ReportIssue)) return false;
            //if (reports.Exists(o => o.Status != (int)ReportStatusType.ReportCollected)) return false;

            var client = new JsonHttpClient(Configs._defaultUrl);

            var pdfs = new List<string>();

            foreach (var thirdDetail in thirdDetailDis)
            {
                if (thirdDetail.ReportKvid == Guid.Empty) continue;
                var report = reports.FirstOrDefault(o => o.Kvid == thirdDetail.ReportKvid);
                if (report == null) continue;
                if (report.Status <= (int)ReportStatusType.ReportIssue) continue;
                #region 报告PDF
                var code = report.Kvid.ToString().Substring(0, 4);
                var internalCode = $"{code}/{report.ReportId}";

                pdfs.Add($"{Configs._pdfUrl}/Lims/Report/{internalCode}");
                #endregion

            }

            try
            {
                var respResult = client.UploadOrderPdf(Configs._token, Configs._secret, third.SerialNumber, pdfs);
                if(respResult!=null&& respResult.Results != null)
                {
                    third.IsUploaded = true;
                    third.AddOnlyProperties(o => o.IsUploaded);
                    conn.UpdateOnly(third);
                    var log = new EntityLog<ThirdReportOrder>();
                    log.OwnerKvid = third.Kvid;
                    log.BizId = third.SerialNumber;
                    log.BizKvid = third.Kvid;
                    log.BizType = third.ThirdType;
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Type = "报告数据回传";
                    log.Summary = $"回传成功";
                    //log.Remark = $"{respResult.Results.Count}";
                    conn.Insert(log);
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception ex)
            {
                var err = ex.GetResponseStatus();
                var message = (err == null ? ex.Message : err.Message);
                var log = new EntityLog<ThirdReportOrder>();
                log.OwnerKvid = third.Kvid;
                log.BizId = third.SerialNumber;
                log.BizKvid = third.Kvid;
                log.BizType = third.ThirdType;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Type = "报告数据回传";
                log.Summary = $"回传失败";
                log.Remark = $"{message}";
                conn.Insert(log);
                return false;
            }
        }
        #endregion
    }

    public class OnlineOrderItem : EntityWithMetadata
    {
        [ApiMember(Description = "订单Kvid")]
        public Guid OrderKvid { get; set; }

        [ApiMember(Description = "商品Kvid")]
        public Guid GoodsKvid { get; set; }

        [ApiMember(Description = "商品分类")]
        public string Category { get; set; }

        [ApiMember(Description = "商品名称")]
        public string GoodsName { get; set; }

        [ApiMember(Description = "SKU编码")]
        public string SkuCode { get; set; }

        [ApiMember(Description = "商品单价")]
        public decimal UnitPrice { get; set; }

        [ApiMember(Description = "购买数量")]
        public int Quantity { get; set; }

        [ApiMember(Description = "商品总价")]
        public decimal TotalPrice { get; set; }

        [ApiMember(Description = "商品规格信息")]
        public string Specifications { get; set; }

        [ApiMember(Description = "商品图片URL")]
        public string ImageUrl { get; set; }

        [ApiMember(Description = "备注")]
        public string Remark { get; set; }

        /// <summary>
        /// 所要创建的项目的kvid，来源于标准库中的检验项目
        /// </summary>
        public List<Guid> DetectionKvids { get; set; }
        public List<string> Detections { get; set; }

        #region 创建/更新人
        [ApiMember(Description = "创建人Kvid")]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid")]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        public string UpdaterName { get; set; }
        #endregion

    }

    public enum OrderStatusType
    {
        /// <summary>
        /// 不支持的状态
        /// </summary>
        Unsupported = 0,

        /// <summary>
        /// 已创建
        /// </summary>
        Created = 100,

        /// <summary>
        /// 已支付
        /// </summary>
        Paid = 200,

        /// <summary>
        /// 已发货
        /// </summary>
        Shipped = 300,

        /// <summary>
        /// 已完成
        /// </summary>
        Completed = 400,

        /// <summary>
        /// 已取消
        /// </summary>
        Cancelled = 500
    }

}
